<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;

class LevelController extends Controller
{
  // Guzzle Http Client
  private $client;

  public function __construct()
  {
    $this->client = new Client(
      [
        'base_uri' => config('app.api_url'),
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'X-Token' => 'xnRxnTHfPlAvfvpF8j5rTcLXKVmkphcz9DwFPZjlQ9DS8vK16mEJfjQlEjPlkZ5e'
        ]
      ]
    );
  }

  /**
   * Display a listing of the resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function index()
  {
    $response = json_decode($this->client->request('GET', 'Levels')->getBody());

    foreach ($response as $level) {
      if ($level->courseId) {
        $course =json_decode($this->client->request('GET', 'Courses/'.$level->courseId)->getBody());
        $level->course = $course;
      }
    }

    return view('level.index')->with([
      'levels' => $response
    ]);
  }

  /**
   * Show the form for creating a new resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function create()
  {
    $response = json_decode($this->client->request('GET', 'Courses')->getBody());

    $courses = [];
    $courses[null] = 'Select Course';
    foreach ($response as $course) {
      $courses[$course->id] = $course->name;
    }

    return view('level.create')->with([
      'courses' => $courses
    ]);
  }

  /**
   * Store a newly created resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @return \Illuminate\Http\Response
   */
  public function store(Request $request)
  {
    $validator = Validator::make($request->all(), [
      'name' => 'required|string|max:191',
      'group' => 'nullable|string',
      'order' => 'nullable|string',
      'image' => 'required|image',
      'difficulty' => 'nullable|string',
      'unlock_cost_in_points' => 'integer|min:0|nullable',
      'unlock_cost_in_gems' => 'integer|min:0|nullable',
      'description' => 'string|nullable',
      'points_awarded_on_completion' => 'required|integer',
      'gems_awarded_on_completion' => 'integer|nullable',
      'decay_rate' => 'regex:/(\d+(\.\d+)?)/|nullable',
      'is_exam' => 'nullable',
      'courseId' => 'string|nullable',
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    // return $request;

    $imageNameWithExt = $request->file('image')->getClientOriginalName();
    $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
    $extension = $request->file('image')->getClientOriginalExtension();
    $imageNameToStore = $imageName.'_'.time().'.'.$extension;

    $body = [
      'name' => $request->name,
      'group' => $request->group,
      'order' => $request->order,
      'image' => $imageNameToStore,
      'difficulty' => $request->difficulty,
      'unlock_cost_in_points' => $request->unlock_cost_in_points,
      'unlock_cost_in_gems' => $request->unlock_cost_in_gems,
      'description' => $request->description,
      'points_awarded_on_completion' => $request->points_awarded_on_completion,
      'gems_awarded_on_completion' => $request->gems_awarded_on_completion,
      'decay_rate' => $request->decay_rate,
      'courseId' => $request->courseId
    ];

    if ($request->has('is_exam')) {
      $body['is_exam'] = true;
    } else {
      $body['is_exam'] = false;
    }

    try {
      $response = $this->client->request('POST', 'Levels', ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        $path = $request->file('image')
        ->storeAs('public/level/images/', $imageNameToStore);
      }
    } catch (\Exception $exception) {
        // return $exception->getMessage();
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('level.index');
  }

  /**
   * Display the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function show($id)
  {
    $response = json_decode($this->client->request('GET', 'Levels/'.$id)->getBody());
    $phrases = json_decode($this->client->request('GET', 'Levels/'.$id.'/phrases')->getBody());

    if ($response->courseId) {
      $course = json_decode($this->client->request('GET', 'Courses/'.$response->courseId)->getBody());
      $response->course = $course;
    }

    return view('level.show')->with([
      'level' => $response,
      'phrases' => $phrases,
    ]);
  }

  /**
   * Show the form for editing the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function edit($id)
  {
    $response = json_decode($this->client->request('GET', 'Courses')->getBody());

    $courses = [];
    $courses[null] = 'Select Course';
    foreach ($response as $course) {
      $courses[$course->id] = $course->name;
    }

    $level = json_decode($this->client->request('GET', 'Levels/'.$id)->getBody());

    // return  $level;
    return view('level.edit')->with([
      'courses' => $courses,
      'level' => $level
    ]);
  }

  /**
   * Update the specified resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function update(Request $request, $id)
  {
    $level = $response = json_decode($this->client->request('GET', 'Levels/'.$id)->getBody());

    $validator = Validator::make($request->all(), [
      'name' => 'required|string|max:191',
      'group' => 'nullable|string',
      'order' => 'nullable|string',
      'image' => 'image',
      'difficulty' => 'nullable|string',
      'unlock_cost_in_points' => 'integer|min:0|nullable',
      'unlock_cost_in_gems' => 'integer|min:0|nullable',
      'description' => 'string|nullable',
      'points_awarded_on_completion' => 'required|integer',
      'gems_awarded_on_completion' => 'integer|nullable',
      'decay_rate' => 'regex:/(\d+(\.\d+)?)/|nullable',
      'is_exam' => 'nullable',
      'courseId' => 'string|nullable',
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    // return $request;

    $imageNameToStore = $level->image;

    if ($request->hasFile('image')) {
      $imageNameWithExt = $request->file('image')->getClientOriginalName();
      $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
      $extension = $request->file('image')->getClientOriginalExtension();
      $imageNameToStore = $imageName.'_'.time().'.'.$extension;
    }

    $body = [
      'name' => $request->name,
      'group' => $request->group,
      'order' => $request->order,
      'image' => $imageNameToStore,
      'difficulty' => $request->difficulty,
      'unlock_cost_in_points' => $request->unlock_cost_in_points,
      'unlock_cost_in_gems' => $request->unlock_cost_in_gems,
      'description' => $request->description,
      'points_awarded_on_completion' => $request->points_awarded_on_completion,
      'gems_awarded_on_completion' => $request->gems_awarded_on_completion,
      'decay_rate' => $request->decay_rate,
      'courseId' => $request->courseId
    ];


    if ($request->has('is_exam')) {
      $body['is_exam'] = true;
    } else {
      $body['is_exam'] = false;
    }

    // return $body;

    try {
      $response = $this->client->request('PUT', 'Levels/'.$id, ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        if ($imageNameToStore != $level->image) {
          Storage::delete('public/level/images/'.$level->image);
          $path = $request->file('image')
          ->storeAs('public/level/images/', $imageNameToStore);
        }
      }
    } catch (\Exception $exception) {
        // return $exception->getMessage();
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('level.show', $id);
  }

  /**
   * Remove the specified resource from storage.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function destroy($id)
  {
    $level = json_decode($this->client->request('GET', 'Levels/'.$id)->getBody());

    $response = $this->client->request('DELETE', 'Levels/'.$id);

    if ($response->getStatusCode() == 200) {
      Storage::delete('public/level/images/'.$level->image);
    }

    return redirect()->route('level.index');
  }
}
