<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Storage;
use GuzzleHttp\Client;

class PhraseController extends Controller
{
  // Guzzle Http Client
  private $client;

  public function __construct()
  {
    $this->client = new Client(
      [
        'base_uri' => config('app.api_url'),
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'X-Token' => 'xnRxnTHfPlAvfvpF8j5rTcLXKVmkphcz9DwFPZjlQ9DS8vK16mEJfjQlEjPlkZ5e'
        ]
      ]
    );
  }

  /**
   * Display a listing of the resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function index()
  {
    $response = json_decode($this->client->request('GET', 'Phrases')->getBody());

    foreach ($response as $phrase) {
      if ($phrase->levelId) {
        $level =json_decode($this->client->request('GET', 'Levels/'.$phrase->levelId)->getBody());
        $phrase->level = $level;
      }
    }
    // return $response;
    return view('phrase.index')->with([
      'phrases' => $response
    ]);;
  }

  /**
   * Show the form for creating a new resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function create()
  {
    $response = json_decode($this->client->request('GET', 'Levels')->getBody());

    $levels = [];
    $levels[null] = 'Select Level';
    foreach ($response as $level) {
      $levels[$level->id] = $level->name;
    }

    return view('phrase.create')->with([
      'levels' => $levels
    ]);
  }

  /**
   * Store a newly created resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @return \Illuminate\Http\Response
   */
  public function store(Request $request)
  {
    // echo '<pre>'.print_r($request, true).'</pre>';
    $validator = Validator::make($request->all(), [
      'name.*.local_word' => 'required|string|max:191|',
      'name.*.english_word' => 'required|string|max:191|',
      // https://github.com/rello/audioplayer/wiki/audio-files-and-mime-types -- Audio Files MIME types
      'name.*.audio' => 'required|mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'name.*.image' => 'required|image',
      'phrase_audio' => 'required|mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'levelId' => 'string|nullable',
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    // return $request;

    $audios = [];
    $images = [];

    foreach($request->name as $i => $phrase) {
      $audioNameWithExt = $phrase['audio']->getClientOriginalName();
      $audioName = pathInfo($audioNameWithExt, PATHINFO_FILENAME);
      $audioExtension = $phrase['audio']->getClientOriginalExtension();
      $audioNameToStore = $audioName.'_'.$i.'_'.time().'.'.$audioExtension;

      array_push($audios, $audioNameToStore);

      $imageNameWithExt = $phrase['image']->getClientOriginalName();
      $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
      $imageExtension = $phrase['image']->getClientOriginalExtension();
      $imageNameToStore = $imageName.'_'.$i.'_'.time().'.'.$imageExtension;

      array_push($images, $imageNameToStore);
    }

    $phraseNameWithExt = $request->file('phrase_audio')->getClientOriginalName();
    $phraseName = pathInfo($phraseNameWithExt, PATHINFO_FILENAME);
    $phraseExtension = $request->file('phrase_audio')->getClientOriginalExtension();
    $phraseNameToStore = 'phrase_audio_'.$phraseName.'_'.time().'.'.$phraseExtension;

    $subphrases = [];
    for ($i=0; $i < count($audios); $i++) {
      array_push($subphrases, [
        'local_word' => $request->name[$i]['local_word'],
        'english_word' => $request->name[$i]['english_word'],
        'audio' => $audios[$i],
        'image' => $images[$i]
      ]);
    }

    // return $subphrases;

    $body = [
      'name' => $subphrases,
      'phrase_audio' => $phraseNameToStore,
      'levelId' => $request->levelId,
    ];

    // return $body;

    try {
      $response = $this->client->request('POST', 'Phrases', ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        foreach($request->name as $i => $phrase) {
          $phrase['audio']->storeAs('public/phrase/audios/', $audios[$i]);
          $phrase['image']->storeAs('public/phrase/images/', $images[$i]);
        }

        $path = $request->file('phrase_audio')
        ->storeAs('public/phrase/phrase/', $phraseNameToStore);
      }
    } catch (\Exception $exception) {
        // return [
        //   'error' => $exception->getMessage()
        // ];
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('phrase.index');
  }

  /**
   * Display the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function show($id)
  {
    $response = json_decode($this->client->request('GET', 'Phrases/'.$id)->getBody());

    if ($response->levelId) {
      $level = json_decode($this->client->request('GET', 'Levels/'.$response->levelId)->getBody());
      $response->level = $level;
    }

    return view('phrase.show')->with([
      'phrase' => $response
    ]);
  }

  /**
   * Show the form for editing the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function edit($id)
  {
    $response = json_decode($this->client->request('GET', 'Levels')->getBody());

    $levels = [];
    $levels[null] = 'Select Level';
    foreach ($response as $level) {
      $levels[$level->id] = $level->name;
    }

    $phrase = json_decode($this->client->request('GET', 'Phrases/'.$id)->getBody());

    return view('phrase.edit')->with([
      'levels' => $levels,
      'phrase' => $phrase
    ]);
  }

  /**
   * Update the specified resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function update(Request $request, $id)
  {
    $validator = Validator::make($request->all(), [
      'name.*.local_word' => 'required|string|max:191|',
      'name.*.english_word' => 'required|string|max:191|',
      // https://github.com/rello/audioplayer/wiki/audio-files-and-mime-types -- Audio Files MIME types
      'name.*.audio' => 'mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'name.*.image' => 'image',
      'phrase_audio' => 'mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'levelId' => 'string|nullable',
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    // return $id;

    $phrases = json_decode($this->client->request('GET', 'Phrases/'.$id)->getBody());

    $audios = [];
    $images = [];

    foreach($request->name as $i => $phrase) {
      $audioNameToStore = '';
      if ($request->hasFile('name.'.$i.'.audio')) {
        $audioNameWithExt = $phrase['audio']->getClientOriginalName();
        $audioName = pathInfo($audioNameWithExt, PATHINFO_FILENAME);
        $audioExtension = $phrase['audio']->getClientOriginalExtension();
        $audioNameToStore = $audioName.'_'.$i.'_'.time().'.'.$audioExtension;
      } else {
        $audioNameToStore = $phrases->name[$i]->audio;
      }

      array_push($audios, $audioNameToStore);

      $imageNameToStore = '';
      if ($request->hasFile('name.'.$i.'.image')) {
        $imageNameWithExt = $phrase['image']->getClientOriginalName();
        $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
        $imageExtension = $phrase['image']->getClientOriginalExtension();
        $imageNameToStore = $imageName.'_'.$i.'_'.time().'.'.$imageExtension;
      } else {
        $imageNameToStore = $phrases->name[$i]->image;
      }

      array_push($images, $imageNameToStore);
    }

    $phraseNameToStore = $phrases->phrase_audio;
    if ($request->hasFile('phrase_audio')) {
      $phraseNameWithExt = $request->file('phrase_audio')->getClientOriginalName();
      $phraseName = pathInfo($phraseNameWithExt, PATHINFO_FILENAME);
      $phraseExtension = $request->file('phrase_audio')->getClientOriginalExtension();
      $phraseNameToStore = 'phrase_audio_'.$phraseName.'_'.time().'.'.$phraseExtension;
    }

    $subphrases = [];
    for ($i=0; $i < count($audios); $i++) {
      array_push($subphrases, [
        'local_word' => $request->name[$i]['local_word'],
        'english_word' => $request->name[$i]['english_word'],
        'audio' => $audios[$i],
        'image' => $images[$i]
      ]);
    }

    // return $subphrases;

    $body = [
      'name' => $subphrases,
      'phrase_audio' => $phraseNameToStore,
      'levelId' => $request->levelId,
    ];

    // return $body;

    try {
      $response = $this->client->request('PUT', 'Phrases/'.$id, ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        foreach($request->name as $i => $phrase) {
          if ($request->hasFile('name.'.$i.'.audio')) {
            $phrase['audio']->storeAs('public/phrase/audios/', $audios[$i]);
          }
          if ($request->hasFile('name.'.$i.'.image')) {
            $phrase['image']->storeAs('public/phrase/images/', $images[$i]);
          }
        }

        foreach($phrases->name as $i => $phrase) {
          $flag = true;
          for ($j = 0; $j < count($body['name']) ; $j++) {
            if (strcmp($phrase->audio, $body['name'][$j]['audio']) == 0) {
              $flag = false;
            }
          }
          if ($flag) {
            Storage::delete('public/phrase/audios/'.$phrase->audio);
          }
        }

        foreach($phrases->name as $i => $phrase) {
          $flag = true;
          for ($j = 0; $j < count($body['name']) ; $j++) {
            if (strcmp($phrase->image, $body['name'][$j]['image']) == 0) {
              $flag = false;
            }
          }
          if ($flag) {
            Storage::delete('public/phrase/images/'.$phrase->image);
          }
        }

        if ($request->hasFile('phrase_audio')) {
          Storage::delete('public/phrase/phrase/'.$phrases->phrase_audio);
          $path = $request->file('phrase_audio')
          ->storeAs('public/phrase/phrase/', $phraseNameToStore);
        }
      }
    } catch (\Exception $exception) {
        // return [
        //   'error' => $exception->getMessage()
        // ];
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('phrase.index');
  }

  /**
   * Remove the specified resource from storage.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function destroy($id)
  {
    $phrase = json_decode($this->client->request('GET', 'Phrases/'.$id)->getBody());

    $response = $this->client->request('DELETE', 'Phrases/'.$id);

    if ($response->getStatusCode() == 200) {
      Storage::delete('public/phrase/phrase/'.$phrase->phrase_audio);
      foreach ($phrase->name as $sub) {
        Storage::delete('public/phrase/audios/'.$sub->audio);
        Storage::delete('public/phrase/images/'.$sub->image);
      }
    }

    return redirect()->route('phrase.index');
  }
}
