<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;

class StoryController extends Controller
{
  private $client;

  public function __construct()
  {
    $this->client = new Client(
      [
        'base_uri' => config('app.api_url'),
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'X-Token' => 'xhHQ9cnZNZixptzC0jucKY8aDeyzSqnYSjsdG8RMiNifSYFGtWvczygQ2AGXJBGU'
        ],
        'query' => [
          'access_token' => 'xhHQ9cnZNZixptzC0jucKY8aDeyzSqnYSjsdG8RMiNifSYFGtWvczygQ2AGXJBGU'
        ]
      ]
    );
  }

  /**
   * Display a listing of the resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function index()
  {
    $response = json_decode($this->client->request('GET', 'Stories')->getBody());

    foreach ($response as $story) {
      if ($story->levelId) {
        $level =json_decode($this->client->request('GET', 'Levels/'.$story->levelId)->getBody());
        $story->level = $level;
      }
    }
    // return $response;

    return view('story.index')->with([
      'stories' => $response
    ]);
  }

  /**
   * Show the form for creating a new resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function create()
  {
    $response = json_decode($this->client->request('GET', 'Levels')->getBody());

    $levels = [];
    $levels[null] = 'Select Level';
    foreach ($response as $level) {
      $levels[$level->id] = $level->name;
    }
    return view('story.create')->with([
      'levels' => $levels
    ]);
  }

  /**
   * Store a newly created resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @return \Illuminate\Http\Response
   */
  public function store(Request $request)
  {
    // return $request->all();

    $validator = Validator::make($request->all(), [
      'scenes.*.background_image' => 'required|image',
      'scenes.*.background_default_view.*' => 'required|integer',
      // https://github.com/rello/audioplayer/wiki/audio-files-and-mime-types -- Audio Files MIME types
      'scenes.*.audio' => 'required|mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'scenes.*.loop_audio' => 'nullable',

      'scenes.*.shots.*.character_type' => 'required',
      'scenes.*.shots.*.character_image' => 'nullable|image',
      'scenes.*.shots.*.name' => 'nullable|string',
      'scenes.*.shots.*.dialogue' => 'nullable|string',
      'scenes.*.shots.*.audio' => 'required|mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'scenes.*.shots.*.background_audio' => 'required|string',
      'scenes.*.shots.*.background_volume' => 'required|integer',
      'scenes.*.shots.*.character_position' => 'required|string',
      'scenes.*.shots.*.background_position.*' => 'required|string',
      'scenes.*.shots.*.background_animation' => 'required|string',

      'levelId' => 'string|nullable',
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    // return $request->all();

    $body = [
      'scene' => []
    ];
    $audios = [];
    $images = [];


    foreach ($request->scenes as $i => $scene) {
      $scn = [];

      $scn['background_default_view'] = $scene['background_default_view'];

      $scn['loop_audio'] = false;
      if (array_key_exists('loop_audio', $scene)) {
        $scn['loop_audio'] = true;
      }

      // Generate Scene audio name
      $audioNameWithExt = $scene['audio']->getClientOriginalName();
      $audioName = pathInfo($audioNameWithExt, PATHINFO_FILENAME);
      $audioExtension = $scene['audio']->getClientOriginalExtension();
      $audioNameToStore = $audioName.'_'.$i.'_'.time().'.'.$audioExtension;

      $scn['audio'] = $audioNameToStore;
      array_push($audios, $audioNameToStore);

      // Generate Background image name
      $imageNameWithExt = $scene['background_image']->getClientOriginalName();
      $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
      $imageExtension = $scene['background_image']->getClientOriginalExtension();
      $imageNameToStore = $imageName.'_'.$i.'_'.time().'.'.$imageExtension;

      $scn['background_image'] = $imageNameToStore;
      array_push($images, $imageNameToStore);

      // array_push($body['scenes'], $scn);

      // $body['scenes']['shots'] = [];

      $scn['shots'] = [];

      foreach ($scene['shots'] as $j => $shot) {
        $s = [];

        $s['character_type'] = $shot['character_type'];
        $s['name'] = $shot['name'];
        $s['dialogue'] = $shot['dialogue'];
        $s['background_audio'] = $shot['background_audio'];
        $s['background_volume'] = $shot['background_volume'];
        $s['character_position'] = $shot['character_position'];
        $s['background_position'] = $shot['background_position'];
        $s['background_animation'] = $shot['background_animation'];

        // Generate Scene audio name
        $audioNameWithExt = $shot['audio']->getClientOriginalName();
        $audioName = pathInfo($audioNameWithExt, PATHINFO_FILENAME);
        $audioExtension = $shot['audio']->getClientOriginalExtension();
        $audioNameToStore = $audioName.'_'.$i.$j.'_'.time().'.'.$audioExtension;

        $s['audio'] = $audioNameToStore;

        // Generate Background image name
        $imageNameWithExt = $shot['character_image']->getClientOriginalName();
        $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
        $imageExtension = $shot['character_image']->getClientOriginalExtension();
        $imageNameToStore = $imageName.'_'.$i.$j.'_'.time().'.'.$imageExtension;

        $s['character_image'] = $imageNameToStore;

        array_push($scn['shots'], $s);
      }
      array_push($body['scene'], $scn);
    }

    $body['levelId'] = $request->levelId;

    // return $body;

    try {
      $response = $this->client->request('POST', 'Stories', ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        foreach($request->scenes as $i => $scene) {
          $scene['audio']
          ->storeAs('public/story/scene/audios/', $body['scene'][$i]['audio']);

          $scene['background_image']
          ->storeAs('public/story/scene/images/', $body['scene'][$i]['background_image']);

          foreach ($scene['shots'] as $j => $shot) {
            $shot['audio']
            ->storeAs('public/story/shot/audios/', $body['scene'][$i]['shots'][$j]['audio']);

            $shot['character_image']
            ->storeAs('public/story/shot/images/', $body['scene'][$i]['shots'][$j]['character_image']);
          }
        }

        return redirect()->route('story.index');
      }
    } catch (\Exception $exception) {
        // return [
        //   'error' => $exception->getMessage()
        // ];
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }
  }

  /**
   * Display the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function show($id)
  {

    $response = json_decode($this->client->request('GET', 'Stories/'.$id)->getBody());

    if ($response->levelId) {
      $level =json_decode($this->client->request('GET', 'Levels/'.$response->levelId)->getBody());
      $response->level = $level;
    }

    return view('story.show')->with([
      'story' => $response
    ]);
  }

  /**
   * Show the form for editing the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function edit($id)
  {
    $response = json_decode($this->client->request('GET', 'Levels')->getBody());

    $levels = [];
    $levels[null] = 'Select Level';
    foreach ($response as $level) {
      $levels[$level->id] = $level->name;
    }

    $response = json_decode($this->client->request('GET', 'Stories/'.$id)->getBody());

    return view('story.edit')->with([
      'story' => $response,
      'levels' => $levels
    ]);
  }

  /**
   * Update the specified resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function update(Request $request, $id)
  {
    // return $request->all();

    $validator = Validator::make($request->all(), [
      'scenes.*.background_image' => 'nullable|image',
      'scenes.*.background_default_view.*' => 'required|integer',
      // https://github.com/rello/audioplayer/wiki/audio-files-and-mime-types -- Audio Files MIME types
      'scenes.*.audio' => 'nullable|mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'scenes.*.loop_audio' => 'nullable',

      'scenes.*.shots.*.character_type' => 'required',
      'scenes.*.shots.*.character_image' => 'nullable|image',
      'scenes.*.shots.*.name' => 'nullable|string',
      'scenes.*.shots.*.dialogue' => 'nullable|string',
      'scenes.*.shots.*.audio' => 'nullable|mimetypes:audio/flac,audio/mpeg,audio/ogg,audio/wav,audio/mid,audio/mp4',
      'scenes.*.shots.*.background_audio' => 'required|string',
      'scenes.*.shots.*.background_volume' => 'required|integer',
      'scenes.*.shots.*.character_position' => 'required|string',
      'scenes.*.shots.*.background_position.*' => 'required|string',
      'scenes.*.shots.*.background_animation' => 'required|string',

      'levelId' => 'string|nullable',
    ]);

    if ($validator->fails()) {
      return $validator->messages();
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }

    // return $request->all();

    $story = json_decode($this->client->request('GET', 'Stories/'.$id)->getBody());

    $body = [
      'scene' => []
    ];

    foreach ($request->scenes as $i => $scene) {
      $scn = [];

      $scn['background_default_view'] = $scene['background_default_view'];

      $scn['loop_audio'] = false;
      if (array_key_exists('loop_audio', $scene)) {
        $scn['loop_audio'] = true;
      }

      // Generate Scene audio name
      $audioNameToStore = '';
      if ($request->hasFile('scenes.'.$i.'.audio')) {
        $audioNameWithExt = $scene['audio']->getClientOriginalName();
        $audioName = pathInfo($audioNameWithExt, PATHINFO_FILENAME);
        $audioExtension = $scene['audio']->getClientOriginalExtension();
        $audioNameToStore = $audioName.'_'.$i.'_'.time().'.'.$audioExtension;
      } else {
        $audioNameToStore = $story->scene[$i]->audio;
      }

      $scn['audio'] = $audioNameToStore;

      // Generate Background image name
      $imageNameToStore = '';
      if ($request->hasFile('scenes.'.$i.'.background_image')) {
        $imageNameWithExt = $scene['background_image']->getClientOriginalName();
        $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
        $imageExtension = $scene['background_image']->getClientOriginalExtension();
        $imageNameToStore = $imageName.'_'.$i.'_'.time().'.'.$imageExtension;
      } else {
        $imageNameToStore = $story->scene[$i]->background_image;
      }

      $scn['background_image'] = $imageNameToStore;

      $scn['shots'] = [];

      foreach ($scene['shots'] as $j => $shot) {
        $s = [];

        $s['character_type'] = $shot['character_type'];
        $s['name'] = $shot['name'];
        $s['dialogue'] = $shot['dialogue'];
        $s['background_audio'] = $shot['background_audio'];
        $s['background_volume'] = $shot['background_volume'];
        $s['character_position'] = $shot['character_position'];
        $s['background_position'] = $shot['background_position'];
        $s['background_animation'] = $shot['background_animation'];

        // Generate Shot audio name
        $audioNameToStore = '';
        if ($request->hasFile('scenes.'.$i.'.shots.'.$j.'.audio')) {
          $audioNameWithExt = $shot['audio']->getClientOriginalName();
          $audioName = pathInfo($audioNameWithExt, PATHINFO_FILENAME);
          $audioExtension = $shot['audio']->getClientOriginalExtension();
          $audioNameToStore = $audioName.'_'.$i.$j.'_'.time().'.'.$audioExtension;
        } else {
          $audioNameToStore = $story->scene[$i]->shots[$j]->audio;
        }

        $s['audio'] = $audioNameToStore;

        // Generate Shot charater image name
        $imageNameToStore = '';
        if ($request->hasFile('scenes.'.$i.'.shots.'.$j.'.character_image')) {
          $imageNameWithExt = $shot['character_image']->getClientOriginalName();
          $imageName = pathInfo($imageNameWithExt, PATHINFO_FILENAME);
          $imageExtension = $shot['character_image']->getClientOriginalExtension();
          $imageNameToStore = $imageName.'_'.$i.$j.'_'.time().'.'.$imageExtension;
        } else {
          $imageNameToStore = $story->scene[$i]->shots[$j]->character_image;
        }

        $s['character_image'] = $imageNameToStore;

        array_push($scn['shots'], $s);
      }
      array_push($body['scene'], $scn);
    }

    $body['levelId'] = $request->levelId;

    // return $body;

    try {
      $response = $this->client->request('PUT', 'Stories/'.$id, ['json' => $body]);

      if ($response->getStatusCode() == 200) {
        foreach($request->scenes as $i => $scene) {
          if ($request->hasFile('scenes.'.$i.'.audio')) {
            $scene['audio']
            ->storeAs('public/story/scene/audios/', $body['scene'][$i]['audio']);
            Storage::delete('public/story/scene/audios/'.$story->scene[$i]->audio);
          }
          if ($request->hasFile('scenes.'.$i.'.background_image')) {
            $scene['background_image']
            ->storeAs('public/story/scene/images/', $body['scene'][$i]['background_image']);
            Storage::delete('public/story/scene/images/'.$story->scene[$i]->background_image);
          }

          foreach ($scene['shots'] as $j => $shot) {
            if ($request->hasFile('scenes.'.$i.'.shots.'.$j.'.audio')) {
              $shot['audio']
              ->storeAs('public/story/shot/audios/', $body['scene'][$i]['shots'][$j]['audio']);
              Storage::delete('public/story/shot/audios/'.$story->scene[$i]->shots[$j]->audio);
            }
            if ($request->hasFile('scenes.'.$i.'.shots.'.$j.'.character_image')) {
              $shot['character_image']
              ->storeAs('public/story/shot/images/', $body['scene'][$i]['shots'][$j]['character_image']);
              Storage::delete('public/story/shot/images/'.$story->scene[$i]->shots[$j]->character_image);
            }
          }
        }

        return redirect()->route('story.show', ['id' => $id]);
      }
    } catch (\Exception $exception) {
        return [
          'error' => $exception->getMessage()
        ];
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }
  }

  /**
   * Remove the specified resource from storage.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function destroy($id)
  {
    $story = json_decode($this->client->request('GET', 'Stories/'.$id)->getBody());

    $response = $this->client->request('DELETE', 'Stories/'.$id);

    if ($response->getStatusCode() == 200) {
      foreach ($story->scene as $scene) {
        Storage::delete('public/story/scene/audios/'.$scene->audio);
        Storage::delete('public/story/scene/images/'.$scene->background_image);
        foreach ($scene->shots as $shot) {
          Storage::delete('public/story/shot/audios/'.$shot->audio);
          Storage::delete('public/story/shot/images/'.$shot->character_image);
        }
      }
    }

    return redirect()->route('story.index');
  }
}
