<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use GuzzleHttp\Client;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Storage;
class UserController extends Controller
{
  // Guzzle Http Client
  private $client;

  public function __construct()
  {
    $this->client = new Client(
      [
        'base_uri' => config('app.api_url'),
        'headers' => [
          'Content-Type' => 'application/json',
          'Accept' => 'application/json',
          'X-Token' => 'xhHQ9cnZNZixptzC0jucKY8aDeyzSqnYSjsdG8RMiNifSYFGtWvczygQ2AGXJBGU'
        ],
        'query' => [
          'access_token' => 'xhHQ9cnZNZixptzC0jucKY8aDeyzSqnYSjsdG8RMiNifSYFGtWvczygQ2AGXJBGU'
        ]
      ]
    );
  }

  /**
   * Display a listing of the resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function index()
  {
    return view('user.index');
  }

  /**
   * Show the form for creating a new resource.
   *
   * @return \Illuminate\Http\Response
   */
  public function create()
  {
    return view('user.create');
  }

  /**
   * Store a newly created resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @return \Illuminate\Http\Response
   */
  public function store(Request $request)
  {
    $validator = Validator::make($request->all(), [
      'realm' => 'nullable|string|max:191',
      'username' => 'nullable|string|max:191',
      'email' => 'required|email',
      'password' => 'confirmed|min:6|required'
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }
    // return $request->all();

    $body = [
      'realm' => $request->realm,
      'username' => $request->username,
      'email' => $request->email,
      'emailVerified ' => $request->emailVerified,
      'password' => $request->password,
    ];

    try {
      $response = $this->client->request('POST', 'Users', ['json' => $body]);
    } catch (\Exception $exception) {
        // return $exception->getMessage();
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('user.index');
  }

  /**
   * Display the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function show($id)
  {
    //
  }

  /**
   * Show the form for editing the specified resource.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function edit($id)
  {
    //
  }

  /**
   * Update the specified resource in storage.
   *
   * @param  \Illuminate\Http\Request  $request
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function update(Request $request, $id)
  {
    //
  }

  /**
   * Remove the specified resource from storage.
   *
   * @param  int  $id
   * @return \Illuminate\Http\Response
   */
  public function destroy($id)
  {
    //
  }

  /**
   * Show form to verify user
   *
   * @return \Illuminate\Http\Response
   */
  public function verify()
  {
    return view('user.verify');
  }

  /**
   * Verify a user
   *
   * @return \Illuminate\Http\Response
   */
  public function verifed(Request $request)
  {
    $validator = Validator::make($request->all(), [
      'id' => 'required',
    ]);

    if ($validator->fails()) {
      Session::flash('error', $validator->messages()->first());
      return redirect()->back()->withErrors($validator)->withInput();
    }
    // return $request->all();

    try {
      $response = $this->client->request('GET', 'Users/'.$request->id);

      if ($response->getStatusCode() == 200) {
        $data = json_decode($response->getBody(), true);
        $body = ['emailVerified' => true];
        try {
          $res = $this->client->request('PATCH', 'Users/'.$request->id, ['json' => $body]);
          if ($res->getStatusCode() == 200) {
            Session::flash('success', 'Email is verified!');
          }
        } catch (\Exception $exception) {
          return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
        }
      }
    } catch (\Exception $exception) {
        return redirect()->back()->withErrors(['error' => $exception->getMessage()])->withInput();
    }

    return redirect()->route('user.verify');
  }
}
