@extends('layouts.app')

@section('content')

@include('story.header')

<section class="">
  <div class="row">
    <div class="col s12">
      <div class="card white">
        <div class="card-content white">
          <p class="card-title inline valign-wrapper">
            <span># <small>{{ $story->id }}</small></span>
          </p>
          <hr>
          <div class="row">
            <div class="col s12">
              <div class="row">
                <div class="col s12 m4">
                  Total Scenes: {{ count($story->scene) }}
                </div>
                <div class="col s12 m4">
                  @php
                    $count = 0;
                    foreach ($story->scene as $scene) {
                      $count += count($scene->shots);
                    }
                  @endphp
                  Total Shots: {{ $count }}
                </div>
                <div class="col s12 m4">
                  @isset($story->level)
                  Level: <a href="{{ route('level.show', $story->level->id) }}">{{ $story->level->name }}</a>
                  @endisset
                </div>
              </div>

              <div class="row">
                <div class="col s12">
                  @foreach ($story->scene as $i => $scene)

                  <div class="scenes">

                    <div class="scene hoverable blue-grey lighten-5">

                      <p class="chip blue white-text m-b-2">Scene {{ $i }}</p>

                      <div class="row">
                        <div class="col s12 m6">
                          <img src="{{ asset('storage/story/scene/images/'.$scene->background_image) }}" alt="" width="80">
                        </div>
                        <div class="col s12 m6">
                          <button type="button" class="btn-floating btn waves-effect waves-light red btn-audio" data-audio="{{ 'scene_audio'.$i.'_'.$loop->iteration }}">
                            <i class="material-icons">play_arrow</i>
                          </button>
                          <audio id="{{ 'scene_audio'.$i.'_'.$loop->iteration }}">
                            <source src="{{ asset('storage/story/scene/audios/'.$scene->audio) }}" >
                            Your browser does not support the audio element.
                          </audio>
                        </div>
                      </div>

                      <div class="row">
                        <p class="col s12"><small>Background Position</small></p>

                        <div class="col s6 m3">
                          x1: {{ $scene->background_default_view[0] }}
                        </div>
                        <div class="col s6 m3">
                          y1: {{ $scene->background_default_view[1] }}
                        </div>
                        <div class="col s6 m3">
                          x2: {{ $scene->background_default_view[2] }}
                        </div>
                        <div class="col s6 m3">
                          y2: {{ $scene->background_default_view[3] }}
                        </div>
                      </div>

                      <div class="switch">
                        <label>
                          Loop Off
                          <input type="checkbox" disabled @if ($scene->loop_audio) checked @endif >
                          <span class="lever"></span>
                          Loop On
                        </label>
                      </div>

                      @foreach ($scene->shots as $j => $shot)

                      <ul class="collapsible expandable shots">
                        <li class="active">
                          <div class="collapsible-header">Shot {{ $j }}</div>
                          <div class="collapsible-body grey lighten-5">
                            <div class="shot">

                              <p class="chip green white-text m-b-2">Shot {{ $j }}</p>

                              <div class="row">
                                <div class="col s12 m6">
                                  Character Type: {{ $shot->character_type }}
                                </div>

                                <div class="col s12 m6">
                                  <img src="{{ asset('storage/story/shot/images/'.$shot->character_image) }}" alt="" width="80">
                                </div>
                              </div>

                              <div class="row">
                                <div class="col s12 m6">
                                  Shot name: {{ $shot->name }}
                                </div>

                                <div class="col s12 m6">
                                  Dialogue: {{ $shot->dialogue }}
                                </div>
                              </div>

                              <div class="row">
                                <div class="col s12 m6">
                                  <button type="button" class="btn-floating btn waves-effect waves-light red btn-audio" data-audio="{{ 'shot_audio'.$i.$j.'_'.$loop->iteration }}">
                                    <i class="material-icons">play_arrow</i>
                                  </button>
                                  <audio id="{{ 'shot_audio'.$i.$j.'_'.$loop->iteration }}">
                                    <source src="{{ asset('storage/story/shot/audios/'.$shot->audio) }}" >
                                    Your browser does not support the audio element.
                                  </audio>
                                </div>

                                <div class="col s12 m6">
                                  Background Audio: {{ $shot->background_audio }}
                                </div>
                              </div>

                              <div class="row">
                                <div class="col s12 m6">
                                  Background Volume: {{ $shot->background_volume }}
                                </div>
                                <div class="col s12 m6">
                                  Character Position: {{ $shot->character_position }}
                                </div>
                              </div>

                              <div class="row">
                                <p class="col s12"><small>Background Position</small></p>

                                <div class="col s6 m3">
                                  x1: {{ $shot->background_position[0] }}
                                </div>
                                <div class="col s6 m3">
                                  y1: {{ $shot->background_position[1] }}
                                </div>
                                <div class="col s6 m3">
                                  x2: {{ $shot->background_position[2] }}
                                </div>
                                <div class="col s6 m3">
                                  y2: {{ $shot->background_position[3] }}
                                </div>
                              </div>

                              <div class="row">
                                <div class="col s12">
                                  Background Animation: {{ $shot->background_animation }}
                                </div>
                              </div>
                            </div>

                          </div>
                        </li>
                      </ul>

                      @endforeach

                    </div>
                  </div>

                  @endforeach

                </div>
              </div>

            </div>
          </div>
        </div>
      </div>
    </div>

  </div>
</section>
@endsection
